@@ -5,6 +5,7 @@ import re, os, csv, json
 from typing import Optional, List, Dict, Union, Any, Callable, Tuple
 from .mutation import Mutation
 
+
 class MutantScorer:
     """
     Copy pasted from PI4KA <- GNB2 <- SnoopCatcher
@@ -51,73 +52,114 @@ class MutantScorer:
     # ===== score =========================================================================
     def score_mutation(self,
                        mutation_name: str,
-                       chain: str,
+                       chains: str,
                        distance: int,
                        cycles: int,
                        interfaces,
                        ref_interface_dG: Dict[str, float],
                        final_func: Optional[Callable] = None,
-                       preminimise: bool = False) -> Tuple[Dict[str, float], pyrosetta.Pose, pyrosetta.Pose]:
+                       preminimise: bool = False,
+                       movement: bool = False) -> Tuple[Dict[str, float], pyrosetta.Pose, pyrosetta.Pose]:
         """
         Scores the mutation ``mutation_name`` (str or Mutation instance)
         returning three objects: a dict of scores, the wt (may differ from pose if preminimise=True) and mutant pose
 
         :param mutation_name:
-        :param chain:
+        :param chains:
         :param distance:
         :param cycles:
         :param interfaces:
-        :param ref_interface_dG:
+        :param ref_interface_dG: premade if no preminimise.
         :param final_func:
         :param preminimise:
         :return:
         """
-        mutation = self.parse_mutation(mutation_name, chain)
-        if self.verbose:
-            print(mutation)
-        if not self.does_contain(mutation):
-            raise ValueError('Absent')
         if preminimise:
             premutant = self.pose.clone()
-            self.relax_around_mover(premutant,
-                                    mutation=mutation,
-                                    distance=distance,
-                                    cycles=cycles)
-            if self.verbose:
-                print('preminimisation complete')
         else:
             premutant = self.pose
-        n = self.scorefxn(premutant)
-        variant = self.make_mutant(premutant,
-                                   mutation=mutation,
-                                   distance=distance,
-                                   cycles=cycles)
+        assert len(chains) > 0, 'Specify at least one chain e.g. "A"'
+        for chain in chains:
+            mutation = self.parse_mutation(mutation_name, chain)
+            if self.verbose:
+                print(mutation)
+            if not self.does_contain(mutation):
+                raise ValueError('Absent')
+            if preminimise:
+                self.relax_around_mover(premutant,
+                                        mutation=mutation,
+                                        distance=distance,
+                                        cycles=cycles)
+                if self.verbose:
+                    print('preminimisation complete')
+        variant = premutant.clone()
+        for chain in chains:
+            mutation = self.parse_mutation(mutation_name, chain)
+            variant = self.make_mutant(variant,
+                                       mutation=mutation,
+                                       distance=distance,
+                                       cycles=cycles,
+                                       inplace=True)
         if self.verbose:
             print('mutant made')
         variant.dump_scored_pdb(f'{self.output_folder}/{self.modelname}.{mutation}.pdb', self.scorefxn)
-        m = self.scorefxn(variant)
+        # score proper
+        data = self.score_only(variant=variant,
+                               reference=premutant,
+                               mutation=mutation,
+                               chains=chains,
+                               distance=distance,
+                               interfaces=interfaces,
+                               ref_interface_dG=ref_interface_dG if not preminimise else dict(),
+                               final_func=final_func,
+                               movement=movement)
+        # if ref_interface_dG (above) is empty score_only calcutates it, so it makes no diff why its empty.
+        # ie. the ref_interface_dG variable here is empty or an empty argument is passed.
+        return data, premutant, variant
+
+    def score_only(self,
+                   variant: pyrosetta.Pose,
+                   reference: pyrosetta.Pose,
+                   mutation: Mutation,
+                   chains: str,
+                   distance: int,
+                   interfaces: List[Tuple[str, str]],
+                   ref_interface_dG: Dict,
+                   final_func: Optional[Callable] = None,
+                   movement: bool=False) -> dict:
+        wt_score = self.scorefxn(reference)
+        mut_score = self.scorefxn(variant)
+        neigh_vector = self.get_neighbour_vector(pose=reference,
+                                                 resi=mutation.pose_resi,
+                                                 chain=None,  # if chain is present, it assumes resi is pdb number.
+                                                 distance=distance)
+        neigh_wt_score = self.scorefxn.get_sub_score(reference, neigh_vector)
+        neigh_mut_score = self.scorefxn.get_sub_score(variant, neigh_vector)
         data = {'model': self.modelname,
                 'mutation': str(mutation),
-                'complex_ddG': m - n,
-                'complex_native_dG': n,
-                'complex_mutant_dG': m,
+                'complex_ddG': mut_score - wt_score,
+                'complex_native_dG': wt_score,
+                'complex_mutant_dG': mut_score,
+                'complex_ddG_neigh_only': neigh_mut_score - neigh_wt_score,
+                'complex_native_dG_only': neigh_wt_score,
+                'complex_mutant_dG_only': neigh_mut_score,
                 'FA_RMSD': self.FA_RMSD(self.pose,
                                         variant,
                                         resi=mutation.pose_resi,
-                                        chain=None,
+                                        chain=None,   # None becase pose_resi is provided.
                                         distance=distance),
                 'CA_RMSD': self.CA_RMSD(self.pose,
                                         variant,
                                         resi=mutation.pose_resi,
-                                        chain=None,
+                                        chain=None,  # None because pose_resi is provided.
                                         distance=distance)
                 }
         # interfaces
         for interface_name, interface_scheme in interfaces:
             if self.has_interface(variant, interface_scheme):
-                if preminimise:
-                    ref_interface_dG[interface_name] = self.score_interface(premutant, interface_scheme)[
-                        'interface_dG']
+                if interface_name not in ref_interface_dG:
+                    ref_interface_dG[interface_name] = self.score_interface(reference,
+                                                                            interface_scheme)['interface_dG']
                 if self.verbose:
                     print(f'{interface_name} ({interface_scheme}) applicable to {self.modelname}')
                 i = self.score_interface(variant, interface_scheme)['interface_dG']
@@ -130,34 +172,36 @@ class MutantScorer:
         if self.verbose:
             print('interface scored')
         # raw
-        wt_scoredex = self.get_wscoredict(premutant)
+        wt_scoredex = self.get_wscoredict(reference)
         mut_scoredex = self.get_wscoredict(variant)
         delta_scoredex = self.delta_scoredict(mut_scoredex, wt_scoredex)
         if self.verbose:
             print('scores stored')
         # movement
-        data['wt_rmsd'] = self.movement(original=premutant, resi=mutation.pdb_resi, chain=chain, distance=distance)
-        data['mut_rmsd'] = self.movement(original=premutant, resi=mutation.pdb_resi, chain=chain, distance=distance)
-        data['ratio_rmsd'] = data['mut_rmsd'] / data['wt_rmsd']
-        if self.verbose:
-            print('movement assessed')
+        if movement:
+            data['wt_rmsd'] = self.movement(original=reference, resi=mutation.pdb_resi, chain=chains[0], distance=distance)
+            data['mut_rmsd'] = self.movement(original=reference, resi=mutation.pdb_resi, chain=chains[0], distance=distance)
+            data['ratio_rmsd'] = data['mut_rmsd'] / data['wt_rmsd']
+            if self.verbose:
+                print('movement assessed')
         data = {**data,
                 **self.prefix_dict(wt_scoredex, 'wt'),
                 **self.prefix_dict(mut_scoredex, 'mut'),
                 **self.prefix_dict(delta_scoredex, 'delta')}
         if final_func is not None:  # callable
-            final_func(data, premutant, variant)
+            final_func(data, reference, variant)
             if self.verbose:
                 print('extra step done.')
-        return data, premutant, variant
+        return data
 
     def make_output_folder(self):
         if not os.path.exists(self.output_folder):
             os.mkdir(self.output_folder)
 
+
     def score_mutations(self,
                         mutations,
-                        chain='A',
+                        chains='A',
                         interfaces=(),  # list of two: name, scheme
                         preminimise=False,
                         distance=10,
@@ -178,7 +222,7 @@ class MutantScorer:
         for mutation_name in mutations:
             try:
                 datum, premutant, mutant = self.score_mutation(mutation_name=mutation_name,
-                                                               chain=chain,
+                                                               chains=chains,
                                                                distance=distance,
                                                                cycles=cycles,
                                                                interfaces=interfaces,
@@ -217,19 +261,25 @@ class MutantScorer:
                     mutation: Union[str, Mutation],
                     chain='A',
                     distance: int = 10,
-                    cycles: int = 5
+                    cycles: int = 5,
+                    inplace: bool = False,
                     ) -> pyrosetta.Pose:
         """
         Make a point mutant (``A23D``).
         :param pose: pose
         :param mutation:
         :param chain:
-        :return:
+        :param inplace:
+        :return: a copy if inplace is false
         """
-        if pose is None:
+        if pose is None and inplace:
+            raise ValueError('inplace parameter = True works only if pose is provide')
+        elif pose is None:
             mutant = self.pose.clone()
-        else:
+        elif not inplace:
             mutant = pose.clone()
+        else:
+            mutant = pose
         if isinstance(mutation, str):
             mutation = Mutation(mutation, chain, mutant)
         MutateResidue = pyrosetta.rosetta.protocols.simple_moves.MutateResidue
@@ -271,7 +321,7 @@ class MutantScorer:
             raise ValueError('mutation or resi+chain required')
         elif mutation is not None:
             resi = mutation.pose_resi
-            chain = None
+            chain = None  # chain None is for pose resi
         else:
             pass
         if pose is None:
@@ -332,13 +382,18 @@ class MutantScorer:
                 'interface_delta_sasa': ia.get_interface_delta_sasa()}
 
     def has_interface(self, pose: pyrosetta.Pose, interface: str) -> bool:
-        if pose is None:
-            pose = self.pose
-        pose2pdb = pose.pdb_info().pose2pdb
-        have_chains = {pose2pdb(r).split()[1] for r in range(1, pose.total_residue() + 1)}
+        have_chains = self.get_present_chains(pose)
         want_chains = set(interface.replace('_', ''))
         return have_chains == want_chains
 
+    def get_present_chains(self, pose: Optional[pyrosetta.Pose] = None):
+        if pose is None:
+            pose = self.pose
+        # pose2pdb = pose.pdb_info().pose2pdb
+        # return {pose2pdb(r).split()[1] for r in range(1, pose.total_residue() + 1)}
+        pdb_info = pose.pdb_info()
+        return {pdb_info.chain(res + 1) for res in range(pose.total_residue())}
+
     def has_residue(self, pose: pyrosetta.Pose, resi: int, chain: str) -> bool:
         if pose is None:
             pose = self.pose
@@ -356,12 +411,12 @@ class MutantScorer:
             x.append(w)
         return x
 
-    def CA_RMSD(self, poseA: pyrosetta.Pose, poseB: pyrosetta.Pose, resi: int, chain: str, distance: int) -> float:
+    def CA_RMSD(self, poseA: pyrosetta.Pose, poseB: pyrosetta.Pose, resi: int, chain: Union[str, None], distance: int) -> float:
         n = self.get_neighbour_vector(pose=poseA, resi=resi, chain=chain, distance=distance)
         residues = self.vector2list(n)
         return pyrosetta.rosetta.core.scoring.CA_rmsd(poseA, poseB, residues)
 
-    def FA_RMSD(self, poseA: pyrosetta.Pose, poseB: pyrosetta.Pose, resi: int, chain: str, distance: int) -> float:
+    def FA_RMSD(self, poseA: pyrosetta.Pose, poseB: pyrosetta.Pose, resi: int, chain: Union[str, None], distance: int) -> float:
         n = self.get_neighbour_vector(pose=poseA, resi=resi, chain=chain, distance=distance,
                                       include_focus_in_subset=False)
         residues = self.vector2list(n)
